@@ -37,14 +37,15 @@ FFmpegReader::FFmpegReader(string path)
 	  audio_pts_offset(99999), video_pts_offset(99999), path(path), is_video_seek(true), check_interlace(false),
 	  check_fps(false), enable_seek(true), is_open(false), seek_audio_frame_found(0), seek_video_frame_found(0),
 	  prev_samples(0), prev_pts(0), pts_total(0), pts_counter(0), is_duration_known(false), largest_frame_processed(0),
-	  current_video_frame(0), has_missing_frames(false), num_packets_since_video_frame(0), num_checks_since_final(0), packet(NULL) {
+	  current_video_frame(0), has_missing_frames(false), num_packets_since_video_frame(0), num_checks_since_final(0),
+	  packet(NULL), use_omp_threads(true) {
 
 	// Initialize FFMpeg, and register all formats and codecs
-	av_register_all();
-	avcodec_register_all();
+	AV_REGISTER_ALL
+	AVCODEC_REGISTER_ALL
 
 	// Init cache
-	working_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 30, info.width, info.height, info.sample_rate, info.channels);
+	working_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
 	missing_frames.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
 	final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
 
@@ -58,14 +59,15 @@ FFmpegReader::FFmpegReader(string path, bool inspect_reader)
 		  audio_pts_offset(99999), video_pts_offset(99999), path(path), is_video_seek(true), check_interlace(false),
 		  check_fps(false), enable_seek(true), is_open(false), seek_audio_frame_found(0), seek_video_frame_found(0),
 		  prev_samples(0), prev_pts(0), pts_total(0), pts_counter(0), is_duration_known(false), largest_frame_processed(0),
-		  current_video_frame(0), has_missing_frames(false), num_packets_since_video_frame(0), num_checks_since_final(0), packet(NULL) {
+		  current_video_frame(0), has_missing_frames(false), num_packets_since_video_frame(0), num_checks_since_final(0),
+		  packet(NULL), use_omp_threads(true) {
 
 	// Initialize FFMpeg, and register all formats and codecs
-	av_register_all();
-	avcodec_register_all();
+	AV_REGISTER_ALL
+	AVCODEC_REGISTER_ALL
 
 	// Init cache
-	working_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 30, info.width, info.height, info.sample_rate, info.channels);
+	working_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
 	missing_frames.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
 	final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
 
@@ -123,11 +125,11 @@ void FFmpegReader::Open()
 		for (unsigned int i = 0; i < pFormatCtx->nb_streams; i++)
 		{
 			// Is this a video stream?
-			if (pFormatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO && videoStream < 0) {
+			if (AV_GET_CODEC_TYPE(pFormatCtx->streams[i]) == AVMEDIA_TYPE_VIDEO && videoStream < 0) {
 				videoStream = i;
 			}
 			// Is this an audio stream?
-			if (pFormatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO && audioStream < 0) {
+			if (AV_GET_CODEC_TYPE(pFormatCtx->streams[i]) == AVMEDIA_TYPE_AUDIO && audioStream < 0) {
 				audioStream = i;
 			}
 		}
@@ -142,20 +144,32 @@ void FFmpegReader::Open()
 
 			// Set the codec and codec context pointers
 			pStream = pFormatCtx->streams[videoStream];
-			pCodecCtx = pFormatCtx->streams[videoStream]->codec;
+
+			// Find the codec ID from stream
+			AVCodecID codecId = AV_FIND_DECODER_CODEC_ID(pStream);
+
+			// Get codec and codec context from stream
+			AVCodec *pCodec = avcodec_find_decoder(codecId);
+			pCodecCtx = AV_GET_CODEC_CONTEXT(pStream, pCodec);
 
 			// Set number of threads equal to number of processors (not to exceed 16)
-			pCodecCtx->thread_count = min(OPEN_MP_NUM_PROCESSORS, 16);
+			pCodecCtx->thread_count = min(FF_NUM_PROCESSORS, 16);
 
-			// Find the decoder for the video stream
-			AVCodec *pCodec = avcodec_find_decoder(pCodecCtx->codec_id);
 			if (pCodec == NULL) {
 				throw InvalidCodec("A valid video codec could not be found for this file.", path);
 			}
+
+			// Init options
+			AVDictionary *opts = NULL;
+			av_dict_set(&opts, "strict", "experimental", 0);
+
 			// Open video codec
-			if (avcodec_open2(pCodecCtx, pCodec, NULL) < 0)
+			if (avcodec_open2(pCodecCtx, pCodec, &opts) < 0)
 				throw InvalidCodec("A video codec was found, but could not be opened.", path);
 
+			// Free options
+			av_dict_free(&opts);
+
 			// Update the File Info struct with video details (if a video stream is found)
 			UpdateVideoInfo();
 		}
@@ -168,33 +182,56 @@ void FFmpegReader::Open()
 
 			// Get a pointer to the codec context for the audio stream
 			aStream = pFormatCtx->streams[audioStream];
-			aCodecCtx = pFormatCtx->streams[audioStream]->codec;
+
+			// Find the codec ID from stream
+			AVCodecID codecId = AV_FIND_DECODER_CODEC_ID(aStream);
+
+			// Get codec and codec context from stream
+			AVCodec *aCodec = avcodec_find_decoder(codecId);
+			aCodecCtx = AV_GET_CODEC_CONTEXT(aStream, aCodec);
 
 			// Set number of threads equal to number of processors (not to exceed 16)
-			aCodecCtx->thread_count = min(OPEN_MP_NUM_PROCESSORS, 16);
+			aCodecCtx->thread_count = min(FF_NUM_PROCESSORS, 16);
 
-			// Find the decoder for the audio stream
-			AVCodec *aCodec = avcodec_find_decoder(aCodecCtx->codec_id);
 			if (aCodec == NULL) {
 				throw InvalidCodec("A valid audio codec could not be found for this file.", path);
 			}
+
+			// Init options
+			AVDictionary *opts = NULL;
+			av_dict_set(&opts, "strict", "experimental", 0);
+
 			// Open audio codec
-			if (avcodec_open2(aCodecCtx, aCodec, NULL) < 0)
+			if (avcodec_open2(aCodecCtx, aCodec, &opts) < 0)
 				throw InvalidCodec("An audio codec was found, but could not be opened.", path);
 
+			// Free options
+			av_dict_free(&opts);
+
 			// Update the File Info struct with audio details (if an audio stream is found)
 			UpdateAudioInfo();
 		}
 
+		// Add format metadata (if any)
+		AVDictionaryEntry *tag = NULL;
+		while ((tag = av_dict_get(pFormatCtx->metadata, "", tag, AV_DICT_IGNORE_SUFFIX))) {
+			QString str_key = tag->key;
+			QString str_value = tag->value;
+			info.metadata[str_key.toStdString()] = str_value.trimmed().toStdString();
+		}
+
 		// Init previous audio location to zero
 		previous_packet_location.frame = -1;
 		previous_packet_location.sample_start = 0;
 
 		// Adjust cache size based on size of frame and audio
-		working_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 30, info.width, info.height, info.sample_rate, info.channels);
+		working_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * info.fps.ToDouble() * 2, info.width, info.height, info.sample_rate, info.channels);
 		missing_frames.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
 		final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
 
+		// Initialize OMP threading support
+		use_omp_threads = openshot::IsOMPEnabled();
+
 		// Mark as "open"
 		is_open = true;
 	}
@@ -214,12 +251,12 @@ void FFmpegReader::Close()
 		if (info.has_video)
 		{
 			avcodec_flush_buffers(pCodecCtx);
-			avcodec_close(pCodecCtx);
+			AV_FREE_CONTEXT(pCodecCtx);
 		}
 		if (info.has_audio)
 		{
 			avcodec_flush_buffers(aCodecCtx);
-			avcodec_close(aCodecCtx);
+			AV_FREE_CONTEXT(aCodecCtx);
 		}
 
 		// Clear final cache
@@ -261,12 +298,12 @@ void FFmpegReader::UpdateAudioInfo()
 	info.has_audio = true;
 	info.file_size = pFormatCtx->pb ? avio_size(pFormatCtx->pb) : -1;
 	info.acodec = aCodecCtx->codec->name;
-	info.channels = aCodecCtx->channels;
-	if (aCodecCtx->channel_layout == 0)
-		aCodecCtx->channel_layout = av_get_default_channel_layout( aCodecCtx->channels );
-	info.channel_layout = (ChannelLayout) aCodecCtx->channel_layout;
-	info.sample_rate = aCodecCtx->sample_rate;
-	info.audio_bit_rate = aCodecCtx->bit_rate;
+	info.channels = AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channels;
+	if (AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channel_layout == 0)
+		AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channel_layout = av_get_default_channel_layout( AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channels );
+	info.channel_layout = (ChannelLayout) AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channel_layout;
+	info.sample_rate = AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->sample_rate;
+	info.audio_bit_rate = AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->bit_rate;
 
 	// Set audio timebase
 	info.audio_timebase.num = aStream->time_base.num;
@@ -294,44 +331,56 @@ void FFmpegReader::UpdateAudioInfo()
 		info.video_length = info.duration * info.fps.ToDouble();
 		info.width = 720;
 		info.height = 480;
-
 	}
 
+	// Fix invalid video lengths for certain types of files (MP3 for example)
+	if (info.has_video && ((info.duration * info.fps.ToDouble()) - info.video_length > 60)) {
+		info.video_length = info.duration * info.fps.ToDouble();
+	}
+
+	// Add audio metadata (if any found)
+	AVDictionaryEntry *tag = NULL;
+	while ((tag = av_dict_get(aStream->metadata, "", tag, AV_DICT_IGNORE_SUFFIX))) {
+		QString str_key = tag->key;
+		QString str_value = tag->value;
+		info.metadata[str_key.toStdString()] = str_value.trimmed().toStdString();
+	}
 }
 
 void FFmpegReader::UpdateVideoInfo()
 {
+	if (check_fps)
+		// Already initialized all the video metadata, no reason to do it again
+		return;
+
 	// Set values of FileInfo struct
 	info.has_video = true;
 	info.file_size = pFormatCtx->pb ? avio_size(pFormatCtx->pb) : -1;
-	info.height = pCodecCtx->height;
-	info.width = pCodecCtx->width;
+	info.height = AV_GET_CODEC_ATTRIBUTES(pStream, pCodecCtx)->height;
+	info.width = AV_GET_CODEC_ATTRIBUTES(pStream, pCodecCtx)->width;
 	info.vcodec = pCodecCtx->codec->name;
-	info.video_bit_rate = pFormatCtx->bit_rate;
-	if (!check_fps)
-	{
-		// set frames per second (fps)
-		info.fps.num = pStream->avg_frame_rate.num;
-		info.fps.den = pStream->avg_frame_rate.den;
-	}
+	info.video_bit_rate = (pFormatCtx->bit_rate / 8);
+
+	// set frames per second (fps)
+	info.fps.num = pStream->avg_frame_rate.num;
+	info.fps.den = pStream->avg_frame_rate.den;
 
 	if (pStream->sample_aspect_ratio.num != 0)
 	{
 		info.pixel_ratio.num = pStream->sample_aspect_ratio.num;
 		info.pixel_ratio.den = pStream->sample_aspect_ratio.den;
 	}
-	else if (pCodecCtx->sample_aspect_ratio.num != 0)
+	else if (AV_GET_CODEC_ATTRIBUTES(pStream, pCodecCtx)->sample_aspect_ratio.num != 0)
 	{
-		info.pixel_ratio.num = pCodecCtx->sample_aspect_ratio.num;
-		info.pixel_ratio.den = pCodecCtx->sample_aspect_ratio.den;
+		info.pixel_ratio.num = AV_GET_CODEC_ATTRIBUTES(pStream, pCodecCtx)->sample_aspect_ratio.num;
+		info.pixel_ratio.den = AV_GET_CODEC_ATTRIBUTES(pStream, pCodecCtx)->sample_aspect_ratio.den;
 	}
 	else
 	{
 		info.pixel_ratio.num = 1;
 		info.pixel_ratio.den = 1;
 	}
-
-	info.pixel_format = pCodecCtx->pix_fmt;
+	info.pixel_format = AV_GET_CODEC_PIXEL_FORMAT(pStream, pCodecCtx);
 
 	// Calculate the DAR (display aspect ratio)
 	Fraction size(info.width * info.pixel_ratio.num, info.height * info.pixel_ratio.den);
@@ -378,18 +427,19 @@ void FFmpegReader::UpdateVideoInfo()
 	}
 
 	// Override an invalid framerate
-	if (info.fps.ToFloat() > 120.0f || (info.fps.num == 0 || info.fps.den == 0))
-	{
-		// Set a few important default video settings (so audio can be divided into frames)
-		info.fps.num = 24;
-		info.fps.den = 1;
-		info.video_timebase.num = 1;
-		info.video_timebase.den = 24;
-
-		// Calculate number of frames
-		info.video_length = round(info.duration * info.fps.ToDouble());
+	if (info.fps.ToFloat() > 240.0f || (info.fps.num <= 0 || info.fps.den <= 0) || info.video_length <= 0) {
+		// Calculate FPS, duration, video bit rate, and video length manually
+		// by scanning through all the video stream packets
+		CheckFPS();
 	}
 
+	// Add video metadata (if any)
+	AVDictionaryEntry *tag = NULL;
+	while ((tag = av_dict_get(pStream->metadata, "", tag, AV_DICT_IGNORE_SUFFIX))) {
+		QString str_key = tag->key;
+		QString str_value = tag->value;
+		info.metadata[str_key.toStdString()] = str_value.trimmed().toStdString();
+	}
 }
 
 
@@ -422,55 +472,55 @@ std::shared_ptr<Frame> FFmpegReader::GetFrame(int64_t requested_frame)
 	}
 	else
 	{
-		// Create a scoped lock, allowing only a single thread to run the following code at one time
-		const GenericScopedLock<CriticalSection> lock(getFrameCriticalSection);
+    #pragma omp critical (ReadStream)
+	  {
+			// Check the cache a 2nd time (due to a potential previous lock)
+			if (has_missing_frames)
+			    CheckMissingFrame(requested_frame);
+			frame = final_cache.GetFrame(requested_frame);
+			if (frame) {
+				// Debug output
+				ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::GetFrame", "returned cached frame on 2nd look", requested_frame, "", -1, "", -1, "", -1, "", -1, "", -1);
 
-		// Check the cache a 2nd time (due to a potential previous lock)
-		if (has_missing_frames)
-		    CheckMissingFrame(requested_frame);
-		frame = final_cache.GetFrame(requested_frame);
-		if (frame) {
-			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::GetFrame", "returned cached frame on 2nd look", requested_frame, "", -1, "", -1, "", -1, "", -1, "", -1);
-
-			// Return the cached frame
-			return frame;
-		}
-
-		// Frame is not in cache
-		// Reset seek count
-		seek_count = 0;
-
-		// Check for first frame (always need to get frame 1 before other frames, to correctly calculate offsets)
-		if (last_frame == 0 && requested_frame != 1)
-			// Get first frame
-			ReadStream(1);
-
-		// Are we within X frames of the requested frame?
-		int64_t diff = requested_frame - last_frame;
-		if (diff >= 1 && diff <= 20)
-		{
-			// Continue walking the stream
-			return ReadStream(requested_frame);
-		}
-		else
-		{
-			// Greater than 30 frames away, or backwards, we need to seek to the nearest key frame
-			if (enable_seek)
-				// Only seek if enabled
-				Seek(requested_frame);
-
-			else if (!enable_seek && diff < 0)
-			{
-				// Start over, since we can't seek, and the requested frame is smaller than our position
-				Close();
-				Open();
+				// Return the cached frame
 			}
+			else {
+				// Frame is not in cache
+				// Reset seek count
+				seek_count = 0;
 
-			// Then continue walking the stream
-			return ReadStream(requested_frame);
-		}
+				// Check for first frame (always need to get frame 1 before other frames, to correctly calculate offsets)
+				if (last_frame == 0 && requested_frame != 1)
+					// Get first frame
+					ReadStream(1);
 
+				// Are we within X frames of the requested frame?
+				int64_t diff = requested_frame - last_frame;
+				if (diff >= 1 && diff <= 20)
+				{
+					// Continue walking the stream
+					frame = ReadStream(requested_frame);
+				}
+				else
+				{
+					// Greater than 30 frames away, or backwards, we need to seek to the nearest key frame
+					if (enable_seek)
+						// Only seek if enabled
+						Seek(requested_frame);
+
+					else if (!enable_seek && diff < 0)
+					{
+						// Start over, since we can't seek, and the requested frame is smaller than our position
+						Close();
+						Open();
+					}
+
+					// Then continue walking the stream
+					frame = ReadStream(requested_frame);
+				}
+			}
+	  } //omp critical
+    return frame;
 	}
 }
 
@@ -540,16 +590,16 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame)
 					num_packets_since_video_frame = 0;
 
 					// Check the status of a seek (if any)
-					if (is_seeking)
+						if (is_seeking)
 						#pragma omp critical (openshot_seek)
-						check_seek = CheckSeek(true);
-					else
-						check_seek = false;
+							check_seek = CheckSeek(true);
+						else
+							check_seek = false;
 
-					if (check_seek) {
-						// Jump to the next iteration of this loop
-						continue;
-					}
+						if (check_seek) {
+							// Jump to the next iteration of this loop
+							continue;
+						}
 
 					// Get the AVFrame from the current packet
 					frame_finished = GetAVFrame();
@@ -562,6 +612,12 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame)
 
 						// Process Video Packet
 						ProcessVideoPacket(requested_frame);
+
+						if (!use_omp_threads) {
+							// Wait on each OMP task to complete before moving on to the next one. This slows
+							// down processing considerably, but might be more stable on some systems.
+							#pragma omp taskwait
+						}
 					}
 
 				}
@@ -572,16 +628,16 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame)
 					num_packets_since_video_frame++;
 
 					// Check the status of a seek (if any)
-					if (is_seeking)
+						if (is_seeking)
 						#pragma omp critical (openshot_seek)
-						check_seek = CheckSeek(false);
-					else
-						check_seek = false;
+							check_seek = CheckSeek(false);
+						else
+							check_seek = false;
 
-					if (check_seek) {
-						// Jump to the next iteration of this loop
-						continue;
-					}
+						if (check_seek) {
+							// Jump to the next iteration of this loop
+							continue;
+						}
 
 					// Update PTS / Frame Offset (if any)
 					UpdatePTSOffset(false);
@@ -594,7 +650,6 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame)
 				}
 
 				// Check if working frames are 'finished'
-				bool is_cache_found = false;
 				if (!is_seeking) {
 					// Check for any missing frames
 					CheckMissingFrame(requested_frame);
@@ -604,7 +659,7 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame)
 				}
 
 				// Check if requested 'final' frame is available
-				is_cache_found = (final_cache.GetFrame(requested_frame) != NULL);
+				bool is_cache_found = (final_cache.GetFrame(requested_frame) != NULL);
 
 				// Increment frames processed
 				packets_processed++;
@@ -616,6 +671,7 @@ std::shared_ptr<Frame> FFmpegReader::ReadStream(int64_t requested_frame)
 			} // end while
 
 		} // end omp single
+
 	} // end omp parallel
 
 	// Debug output
@@ -676,28 +732,60 @@ int FFmpegReader::GetNextPacket()
 bool FFmpegReader::GetAVFrame()
 {
 	int frameFinished = -1;
+	int ret = 0;
 
 	// Decode video frame
 	AVFrame *next_frame = AV_ALLOCATE_FRAME();
 	#pragma omp critical (packet_cache)
-	avcodec_decode_video2(pCodecCtx, next_frame, &frameFinished, packet);
-
-	// is frame finished
-	if (frameFinished)
 	{
-		// AVFrames are clobbered on the each call to avcodec_decode_video, so we
-		// must make a copy of the image data before this method is called again.
-		pFrame = new AVPicture();
-		avpicture_alloc(pFrame, pCodecCtx->pix_fmt, info.width, info.height);
-		av_picture_copy(pFrame, (AVPicture *) next_frame, pCodecCtx->pix_fmt, info.width, info.height);
-
-		// Detect interlaced frame (only once)
-		if (!check_interlace)
-		{
-			check_interlace = true;
-			info.interlaced_frame = next_frame->interlaced_frame;
-			info.top_field_first = next_frame->top_field_first;
+	#if IS_FFMPEG_3_2
+		frameFinished = 0;
+		ret = avcodec_send_packet(pCodecCtx, packet);
+		if (ret < 0 || ret == AVERROR(EAGAIN) || ret == AVERROR_EOF) {
+			ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::GetAVFrame (Packet not sent)", "", -1, "", -1, "", -1, "", -1, "", -1, "", -1);
 		}
+		else {
+			pFrame = new AVFrame();
+			while (ret >= 0) {
+				ret =  avcodec_receive_frame(pCodecCtx, next_frame);
+		  if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF) {
+		  break;
+				}
+				// TODO also handle possible further frames
+				// Use only the first frame like avcodec_decode_video2
+				if (frameFinished == 0 ) {
+					frameFinished = 1;
+					av_image_alloc(pFrame->data, pFrame->linesize, info.width, info.height, (AVPixelFormat)(pStream->codecpar->format), 1);
+					av_image_copy(pFrame->data, pFrame->linesize, (const uint8_t**)next_frame->data, next_frame->linesize,
+												(AVPixelFormat)(pStream->codecpar->format), info.width, info.height);
+					if (!check_interlace)	{
+						check_interlace = true;
+						info.interlaced_frame = next_frame->interlaced_frame;
+						info.top_field_first = next_frame->top_field_first;
+					}
+				}
+			}
+		}
+	#else
+		avcodec_decode_video2(pCodecCtx, next_frame, &frameFinished, packet);
+
+		// is frame finished
+		if (frameFinished) {
+			// AVFrames are clobbered on the each call to avcodec_decode_video, so we
+			// must make a copy of the image data before this method is called again.
+			pFrame = AV_ALLOCATE_FRAME();
+			avpicture_alloc((AVPicture *) pFrame, pCodecCtx->pix_fmt, info.width, info.height);
+			av_picture_copy((AVPicture *) pFrame, (AVPicture *) next_frame, pCodecCtx->pix_fmt, info.width,
+							info.height);
+
+			// Detect interlaced frame (only once)
+			if (!check_interlace) {
+				check_interlace = true;
+				info.interlaced_frame = next_frame->interlaced_frame;
+				info.top_field_first = next_frame->top_field_first;
+			}
+		}
+	#endif
 	}
 
 	// deallocate the frame
@@ -734,7 +822,7 @@ bool FFmpegReader::CheckSeek(bool is_video)
 			ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::CheckSeek (Too far, seek again)", "is_video_seek", is_video_seek, "max_seeked_frame", max_seeked_frame, "seeking_frame", seeking_frame, "seeking_pts", seeking_pts, "seek_video_frame_found", seek_video_frame_found, "seek_audio_frame_found", seek_audio_frame_found);
 
 			// Seek again... to the nearest Keyframe
-			Seek(seeking_frame - (20 * seek_count * seek_count));
+			Seek(seeking_frame - (10 * seek_count * seek_count));
 		}
 		else
 		{
@@ -779,11 +867,11 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame)
 	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::ProcessVideoPacket (Before)", "requested_frame", requested_frame, "current_frame", current_frame, "", -1, "", -1, "", -1, "", -1);
 
 	// Init some things local (for OpenMP)
-	PixelFormat pix_fmt = pCodecCtx->pix_fmt;
+	PixelFormat pix_fmt = AV_GET_CODEC_PIXEL_FORMAT(pStream, pCodecCtx);
 	int height = info.height;
 	int width = info.width;
 	int64_t video_length = info.video_length;
-    AVPicture *my_frame = pFrame;
+	AVFrame *my_frame = pFrame;
 
 	// Add video frame to list of processing video frames
 	const GenericScopedLock<CriticalSection> lock(processingCriticalSection);
@@ -823,17 +911,16 @@ void FFmpegReader::ProcessVideoPacket(int64_t requested_frame)
 		}
 
 		// Determine required buffer size and allocate buffer
-		numBytes = avpicture_get_size(PIX_FMT_RGBA, width, height);
+		numBytes = AV_GET_IMAGE_SIZE(PIX_FMT_RGBA, width, height);
+
 		#pragma omp critical (video_buffer)
 		buffer = (uint8_t *) av_malloc(numBytes * sizeof(uint8_t));
 
-		// Assign appropriate parts of buffer to image planes in pFrameRGB
-		// Note that pFrameRGB is an AVFrame, but AVFrame is a superset
-		// of AVPicture
-		avpicture_fill((AVPicture *) pFrameRGB, buffer, PIX_FMT_RGBA, width, height);
+		// Copy picture data from one AVFrame (or AVPicture) to another one.
+		AV_COPY_PICTURE_DATA(pFrameRGB, buffer, PIX_FMT_RGBA, width, height);
 
-		SwsContext *img_convert_ctx = sws_getContext(info.width, info.height, pCodecCtx->pix_fmt, width,
-													  height, PIX_FMT_RGBA, SWS_BILINEAR, NULL, NULL, NULL);
+		SwsContext *img_convert_ctx = sws_getContext(info.width, info.height, AV_GET_CODEC_PIXEL_FORMAT(pStream, pCodecCtx), width,
+															  height, PIX_FMT_RGBA, SWS_LANCZOS, NULL, NULL, NULL);
 
 		// Resize / Convert to RGB
 		sws_scale(img_convert_ctx, my_frame->data, my_frame->linesize, 0,
@@ -903,21 +990,53 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame, int64_t target_fr
 	int data_size = 0;
 
 	// re-initialize buffer size (it gets changed in the avcodec_decode_audio2 method call)
-	int buf_size = AVCODEC_MAX_AUDIO_FRAME_SIZE + FF_INPUT_BUFFER_PADDING_SIZE;
-	int used = avcodec_decode_audio4(aCodecCtx, audio_frame, &frame_finished, packet);
+	int buf_size = AVCODEC_MAX_AUDIO_FRAME_SIZE + MY_INPUT_BUFFER_PADDING_SIZE;
+	#pragma omp critical (ProcessAudioPacket)
+	{
+	#if IS_FFMPEG_3_2
+		int ret = 0;
+		frame_finished = 1;
+		while((packet->size > 0 || (!packet->data && frame_finished)) && ret >= 0) {
+			frame_finished = 0;
+			ret =  avcodec_send_packet(aCodecCtx, packet);
+			if (ret < 0 && ret !=  AVERROR(EINVAL) && ret != AVERROR_EOF) {
+				avcodec_send_packet(aCodecCtx, NULL);
+				break;
+			}
+			if (ret >= 0)
+				packet->size = 0;
+			ret =  avcodec_receive_frame(aCodecCtx, audio_frame);
+			if (ret >= 0)
+				frame_finished = 1;
+			if(ret == AVERROR(EINVAL) || ret == AVERROR_EOF) {
+				avcodec_flush_buffers(aCodecCtx);
+				ret = 0;
+			}
+			if (ret >= 0) {
+				ret = frame_finished;
+			}
+		}
+		if (!packet->data && !frame_finished)
+		{
+			ret = -1;
+		}
+	#else
+		int used = avcodec_decode_audio4(aCodecCtx, audio_frame, &frame_finished, packet);
+#endif
+	}
 
 	if (frame_finished) {
 
 		// determine how many samples were decoded
-		int planar = av_sample_fmt_is_planar(aCodecCtx->sample_fmt);
+		int planar = av_sample_fmt_is_planar((AVSampleFormat)AV_GET_CODEC_PIXEL_FORMAT(aStream, aCodecCtx));
 		int plane_size = -1;
 		data_size = av_samples_get_buffer_size(&plane_size,
-				aCodecCtx->channels,
+				AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channels,
 				audio_frame->nb_samples,
-				aCodecCtx->sample_fmt, 1);
+				(AVSampleFormat)(AV_GET_SAMPLE_FORMAT(aStream, aCodecCtx)), 1);
 
 		// Calculate total number of samples
-		packet_samples = audio_frame->nb_samples * aCodecCtx->channels;
+		packet_samples = audio_frame->nb_samples * AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channels;
 	}
 
 	// Estimate the # of samples and the end of this packet's location (to prevent GAPS for the next timestamp)
@@ -976,9 +1095,9 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame, int64_t target_fr
 
 
 	// Allocate audio buffer
-	int16_t *audio_buf = new int16_t[AVCODEC_MAX_AUDIO_FRAME_SIZE + FF_INPUT_BUFFER_PADDING_SIZE];
+	int16_t *audio_buf = new int16_t[AVCODEC_MAX_AUDIO_FRAME_SIZE + MY_INPUT_BUFFER_PADDING_SIZE];
 
-	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::ProcessAudioPacket (ReSample)", "packet_samples", packet_samples, "info.channels", info.channels, "info.sample_rate", info.sample_rate, "aCodecCtx->sample_fmt", aCodecCtx->sample_fmt, "AV_SAMPLE_FMT_S16", AV_SAMPLE_FMT_S16, "", -1);
+	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::ProcessAudioPacket (ReSample)", "packet_samples", packet_samples, "info.channels", info.channels, "info.sample_rate", info.sample_rate, "aCodecCtx->sample_fmt", AV_GET_SAMPLE_FORMAT(aStream, aCodecCtx), "AV_SAMPLE_FMT_S16", AV_SAMPLE_FMT_S16, "", -1);
 
 	// Create output frame
 	AVFrame *audio_converted = AV_ALLOCATE_FRAME();
@@ -986,23 +1105,23 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame, int64_t target_fr
 	audio_converted->nb_samples = audio_frame->nb_samples;
 	av_samples_alloc(audio_converted->data, audio_converted->linesize, info.channels, audio_frame->nb_samples, AV_SAMPLE_FMT_S16, 0);
 
-	AVAudioResampleContext *avr = NULL;
+	SWRCONTEXT *avr = NULL;
 	int nb_samples = 0;
 
 	// setup resample context
-	avr = avresample_alloc_context();
-	av_opt_set_int(avr,  "in_channel_layout", aCodecCtx->channel_layout, 0);
-	av_opt_set_int(avr, "out_channel_layout", aCodecCtx->channel_layout, 0);
-	av_opt_set_int(avr,  "in_sample_fmt",     aCodecCtx->sample_fmt,     0);
+	avr = SWR_ALLOC();
+	av_opt_set_int(avr,  "in_channel_layout", AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channel_layout, 0);
+	av_opt_set_int(avr, "out_channel_layout", AV_GET_CODEC_ATTRIBUTES(aStream, aCodecCtx)->channel_layout, 0);
+	av_opt_set_int(avr,  "in_sample_fmt",     AV_GET_SAMPLE_FORMAT(aStream, aCodecCtx), 0);
 	av_opt_set_int(avr, "out_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
 	av_opt_set_int(avr,  "in_sample_rate",    info.sample_rate,    0);
 	av_opt_set_int(avr, "out_sample_rate",    info.sample_rate,    0);
 	av_opt_set_int(avr,  "in_channels",       info.channels,    0);
 	av_opt_set_int(avr, "out_channels",       info.channels,    0);
-	int r = avresample_open(avr);
+	int r = SWR_INIT(avr);
 
 	// Convert audio samples
-	nb_samples = avresample_convert(avr, 	// audio resample context
+	nb_samples = SWR_CONVERT(avr, 	// audio resample context
 			audio_converted->data, 			// output data pointers
 			audio_converted->linesize[0], 	// output plane size, in bytes. (0 if unknown)
 			audio_converted->nb_samples,	// maximum number of samples that the output buffer can hold
@@ -1014,8 +1133,8 @@ void FFmpegReader::ProcessAudioPacket(int64_t requested_frame, int64_t target_fr
 	memcpy(audio_buf, audio_converted->data[0], audio_converted->nb_samples * av_get_bytes_per_sample(AV_SAMPLE_FMT_S16) * info.channels);
 
 	// Deallocate resample buffer
-	avresample_close(avr);
-	avresample_free(&avr);
+	SWR_CLOSE(avr);
+	SWR_FREE(&avr);
 	avr = NULL;
 
 	// Free AVFrames
@@ -1167,7 +1286,7 @@ void FFmpegReader::Seek(int64_t requested_frame)
 	}
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::Seek", "requested_frame", requested_frame, "seek_count", seek_count, "last_frame", last_frame, "processing_video_frames_size", processing_video_frames_size, "processing_audio_frames_size", processing_audio_frames_size, "", -1);
+	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::Seek", "requested_frame", requested_frame, "seek_count", seek_count, "last_frame", last_frame, "processing_video_frames_size", processing_video_frames_size, "processing_audio_frames_size", processing_audio_frames_size, "video_pts_offset", video_pts_offset);
 
 	// Wait for any processing frames to complete
 	while (processing_video_frames_size + processing_audio_frames_size > 0) {
@@ -1209,7 +1328,7 @@ void FFmpegReader::Seek(int64_t requested_frame)
 	seek_count++;
 
 	// If seeking near frame 1, we need to close and re-open the file (this is more reliable than seeking)
-	int buffer_amount = 6;
+	int buffer_amount = max(OPEN_MP_NUM_PROCESSORS, 8);
 	if (requested_frame - buffer_amount < 20)
 	{
 		// Close and re-open file (basically seeking to frame 1)
@@ -1241,7 +1360,7 @@ void FFmpegReader::Seek(int64_t requested_frame)
 		{
 			seek_target = ConvertFrameToVideoPTS(requested_frame - buffer_amount);
 			if (av_seek_frame(pFormatCtx, info.video_stream_index, seek_target, AVSEEK_FLAG_BACKWARD) < 0) {
-				fprintf(stderr, "%s: error while seeking video stream\n", pFormatCtx->filename);
+				fprintf(stderr, "%s: error while seeking video stream\n", pFormatCtx->AV_FILENAME);
 			} else
 			{
 				// VIDEO SEEK
@@ -1255,7 +1374,7 @@ void FFmpegReader::Seek(int64_t requested_frame)
 		{
 			seek_target = ConvertFrameToAudioPTS(requested_frame - buffer_amount);
 			if (av_seek_frame(pFormatCtx, info.audio_stream_index, seek_target, AVSEEK_FLAG_BACKWARD) < 0) {
-				fprintf(stderr, "%s: error while seeking audio stream\n", pFormatCtx->filename);
+				fprintf(stderr, "%s: error while seeking audio stream\n", pFormatCtx->AV_FILENAME);
 			} else
 			{
 				// AUDIO SEEK
@@ -1558,7 +1677,6 @@ bool FFmpegReader::CheckMissingFrame(int64_t requested_frame)
 	// Debug output
 	ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::CheckMissingFrame", "requested_frame", requested_frame, "has_missing_frames", has_missing_frames, "missing_video_frames.size()", missing_video_frames.size(), "checked_count", checked_count, "", -1, "", -1);
 
-
 	// Missing frames (sometimes frame #'s are skipped due to invalid or missing timestamps)
 	map<int64_t, int64_t>::iterator itr;
 	bool found_missing_frame = false;
@@ -1639,6 +1757,11 @@ void FFmpegReader::CheckWorkingFrames(bool end_of_stream, int64_t requested_fram
 			// No frames found
 			break;
 
+		// Remove frames which are too old
+		if (f && f->number < (requested_frame - (OPEN_MP_NUM_PROCESSORS * 2))) {
+			working_cache.Remove(f->number);
+		}
+
 		// Check if this frame is 'missing'
 		CheckMissingFrame(f->number);
 
@@ -1685,7 +1808,6 @@ void FFmpegReader::CheckWorkingFrames(bool end_of_stream, int64_t requested_fram
 			}
 
 			if (info.has_audio && !is_audio_ready) {
-				const GenericScopedLock<CriticalSection> lock(processingCriticalSection);
 				// Mark audio as processed, and indicate the frame has audio data
 				is_audio_ready = true;
 			}
@@ -1698,10 +1820,15 @@ void FFmpegReader::CheckWorkingFrames(bool end_of_stream, int64_t requested_fram
 		if ((!end_of_stream && is_video_ready && is_audio_ready) || end_of_stream || is_seek_trash)
 		{
 			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::CheckWorkingFrames (mark frame as final)", "requested_frame", requested_frame, "f->number", f->number, "is_seek_trash", is_seek_trash, "Working Cache Count", working_cache.Count(), "Final Cache Count", final_cache.Count(), "", -1);
+			ZmqLogger::Instance()->AppendDebugMethod("FFmpegReader::CheckWorkingFrames (mark frame as final)", "requested_frame", requested_frame, "f->number", f->number, "is_seek_trash", is_seek_trash, "Working Cache Count", working_cache.Count(), "Final Cache Count", final_cache.Count(), "end_of_stream", end_of_stream);
 
 			if (!is_seek_trash)
 			{
+				// Add missing image (if needed - sometimes end_of_stream causes frames with only audio)
+				if (info.has_video && !is_video_ready && last_video_frame)
+					// Copy image from last frame
+					f->AddImage(std::shared_ptr<QImage>(new QImage(*last_video_frame->GetImage())));
+
 				// Reset counter since last 'final' frame
 				num_checks_since_final = 0;
 
@@ -1742,16 +1869,14 @@ void FFmpegReader::CheckWorkingFrames(bool end_of_stream, int64_t requested_fram
 void FFmpegReader::CheckFPS()
 {
 	check_fps = true;
-	avpicture_alloc(pFrame, pCodecCtx->pix_fmt, info.width, info.height);
 
 	int first_second_counter = 0;
 	int second_second_counter = 0;
 	int third_second_counter = 0;
 	int forth_second_counter = 0;
 	int fifth_second_counter = 0;
-
-	int iterations = 0;
-	int threshold = 500;
+	int frames_detected = 0;
+	int64_t pts = 0;
 
 	// Loop through the stream
 	while (true)
@@ -1771,7 +1896,7 @@ void FFmpegReader::CheckFPS()
 				UpdatePTSOffset(true);
 
 				// Get PTS of this packet
-				int64_t pts = GetVideoPTS();
+				pts = GetVideoPTS();
 
 				// Remove pFrame
 				RemoveAVFrame(pFrame);
@@ -1793,77 +1918,62 @@ void FFmpegReader::CheckFPS()
 					forth_second_counter++;
 				else if (video_seconds > 4.0 && video_seconds <= 5.0)
 					fifth_second_counter++;
-				else
-					// Too far
-					break;
+
+				// Increment counters
+				frames_detected++;
 			}
 		}
-
-		// Increment counters
-		iterations++;
-
-		// Give up (if threshold exceeded)
-		if (iterations > threshold)
-			break;
 	}
 
 	// Double check that all counters have greater than zero (or give up)
-	if (second_second_counter == 0 || third_second_counter == 0 || forth_second_counter == 0 || fifth_second_counter == 0)
-	{
-		// Seek to frame 1
-		Seek(1);
+	if (second_second_counter != 0 && third_second_counter != 0 && forth_second_counter != 0 && fifth_second_counter != 0) {
+		// Calculate average FPS (average of first few seconds)
+		int sum_fps = second_second_counter + third_second_counter + forth_second_counter + fifth_second_counter;
+		int avg_fps = round(sum_fps / 4.0f);
 
-		// exit with no changes to FPS (not enough data to calculate)
-		return;
+		// Update FPS
+		info.fps = Fraction(avg_fps, 1);
+
+		// Update Duration and Length
+		info.video_length = frames_detected;
+		info.duration = frames_detected / (sum_fps / 4.0f);
+
+		// Update video bit rate
+		info.video_bit_rate = info.file_size / info.duration;
+	} else if (second_second_counter != 0 && third_second_counter != 0) {
+		// Calculate average FPS (only on second 2)
+		int sum_fps = second_second_counter;
+
+		// Update FPS
+		info.fps = Fraction(sum_fps, 1);
+
+		// Update Duration and Length
+		info.video_length = frames_detected;
+		info.duration = frames_detected / float(sum_fps);
+
+		// Update video bit rate
+		info.video_bit_rate = info.file_size / info.duration;
+	} else {
+		// Too short to determine framerate, just default FPS
+		// Set a few important default video settings (so audio can be divided into frames)
+		info.fps.num = 30;
+		info.fps.den = 1;
+
+		// Calculate number of frames
+		info.video_length = frames_detected;
+		info.duration = frames_detected / info.fps.ToFloat();
 	}
-
-	int sum_fps = second_second_counter + third_second_counter + forth_second_counter + fifth_second_counter;
-	int avg_fps = round(sum_fps / 4.0f);
-
-	// Sometimes the FPS is incorrectly detected by FFmpeg.  If the 1st and 2nd seconds counters
-	// agree with each other, we are going to adjust the FPS of this reader instance.  Otherwise, print
-	// a warning message.
-
-	// Get diff from actual frame rate
-	double fps = info.fps.ToDouble();
-	double diff = fps - double(avg_fps);
-
-	// Is difference bigger than 1 frame?
-	if (diff <= -1 || diff >= 1)
-	{
-		// Compare to half the frame rate (the most common type of issue)
-		double half_fps = Fraction(info.fps.num / 2, info.fps.den).ToDouble();
-		diff = half_fps - double(avg_fps);
-
-		// Is difference bigger than 1 frame?
-		if (diff <= -1 || diff >= 1)
-		{
-			// Update FPS for this reader instance
-			info.fps = Fraction(avg_fps, 1);
-		}
-		else
-		{
-			// Update FPS for this reader instance (to 1/2 the original framerate)
-			info.fps = Fraction(info.fps.num / 2, info.fps.den);
-		}
-	}
-
-	// Seek to frame 1
-	Seek(1);
 }
 
 // Remove AVFrame from cache (and deallocate it's memory)
-void FFmpegReader::RemoveAVFrame(AVPicture* remove_frame)
+void FFmpegReader::RemoveAVFrame(AVFrame* remove_frame)
 {
     // Remove pFrame (if exists)
     if (remove_frame)
     {
         // Free memory
-        avpicture_free(remove_frame);
-
-        // Delete the object
-        delete remove_frame;
-    }
+		av_freep(&remove_frame->data[0]);
+	}
 }
 
 // Remove AVPacket from cache (and deallocate it's memory)
