@@ -31,8 +31,9 @@ using namespace std;
 using namespace openshot;
 
 // Constructor - blank frame (300x200 blank image, 48kHz audio silence)
-Frame::Frame() : number(1), pixel_ratio(1,1), channels(2), width(1), height(1),
-		channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
+Frame::Frame() : number(1), pixel_ratio(1,1), channels(2), width(1), height(1), color("#000000"),
+		channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false),
+		max_audio_sample(0)
 {
 	// Init the image magic and audio buffer
 	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, 0));
@@ -43,8 +44,9 @@ Frame::Frame() : number(1), pixel_ratio(1,1), channels(2), width(1), height(1),
 
 // Constructor - image only (48kHz audio silence)
 Frame::Frame(int64_t number, int width, int height, string color)
-	: number(number), pixel_ratio(1,1), channels(2), width(width), height(height),
-	  channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
+	: number(number), pixel_ratio(1,1), channels(2), width(width), height(height), color(color),
+	  channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false),
+	  max_audio_sample(0)
 {
 	// Init the image magic and audio buffer
 	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, 0));
@@ -55,8 +57,9 @@ Frame::Frame(int64_t number, int width, int height, string color)
 
 // Constructor - audio only (300x200 blank image)
 Frame::Frame(int64_t number, int samples, int channels) :
-		number(number), pixel_ratio(1,1), channels(channels), width(1), height(1),
-		channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
+		number(number), pixel_ratio(1,1), channels(channels), width(1), height(1), color("#000000"),
+		channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false),
+		max_audio_sample(0)
 {
 	// Init the image magic and audio buffer
 	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, samples));
@@ -67,8 +70,9 @@ Frame::Frame(int64_t number, int samples, int channels) :
 
 // Constructor - image & audio
 Frame::Frame(int64_t number, int width, int height, string color, int samples, int channels)
-	: number(number), pixel_ratio(1,1), channels(channels), width(width), height(height),
-	  channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false)
+	: number(number), pixel_ratio(1,1), channels(channels), width(width), height(height), color(color),
+	  channel_layout(LAYOUT_STEREO), sample_rate(44100), qbuffer(NULL), has_audio_data(false), has_image_data(false),
+	  max_audio_sample(0)
 {
 	// Init the image magic and audio buffer
 	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(channels, samples));
@@ -85,20 +89,33 @@ Frame::Frame ( const Frame &other )
 	DeepCopy(other);
 }
 
+// Assignment operator
+Frame& Frame::operator= (const Frame& other)
+{
+	// copy pointers and data
+	DeepCopy(other);
+
+	return *this;
+}
+
 // Copy data and pointers from another Frame instance
 void Frame::DeepCopy(const Frame& other)
 {
 	number = other.number;
-	image = std::shared_ptr<QImage>(new QImage(*(other.image)));
-	audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(*(other.audio)));
-	pixel_ratio = Fraction(other.pixel_ratio.num, other.pixel_ratio.den);
 	channels = other.channels;
+	width = other.width;
+	height = other.height;
 	channel_layout = other.channel_layout;
 	has_audio_data = other.has_image_data;
 	has_image_data = other.has_image_data;
 	sample_rate = other.sample_rate;
+	pixel_ratio = Fraction(other.pixel_ratio.num, other.pixel_ratio.den);
+	color = other.color;
 
-
+	if (other.image)
+		image = std::shared_ptr<QImage>(new QImage(*(other.image)));
+	if (other.audio)
+		audio = std::shared_ptr<juce::AudioSampleBuffer>(new juce::AudioSampleBuffer(*(other.audio)));
 	if (other.wave_image)
 		wave_image = std::shared_ptr<QImage>(new QImage(*(other.wave_image)));
 }
@@ -162,7 +179,7 @@ std::shared_ptr<QImage> Frame::GetWaveform(int width, int height, int Red, int G
 	QVector<QPointF> labels;
 
 	// Calculate width of an image based on the # of samples
-	int total_samples = audio->getNumSamples();
+	int total_samples = GetAudioSamplesCount();
 	if (total_samples > 0)
 	{
 		// If samples are present...
@@ -180,7 +197,7 @@ std::shared_ptr<QImage> Frame::GetWaveform(int width, int height, int Red, int G
 			// Get audio for this channel
 			const float *samples = audio->getReadPointer(channel);
 
-			for (int sample = 0; sample < audio->getNumSamples(); sample++, X++)
+			for (int sample = 0; sample < GetAudioSamplesCount(); sample++, X++)
 			{
 				// Sample value (scaled to -100 to 100)
 				float value = samples[sample] * 100;
@@ -322,7 +339,7 @@ float* Frame::GetPlanarAudioSamples(int new_sample_rate, AudioResampler* resampl
 	float *output = NULL;
 	AudioSampleBuffer *buffer(audio.get());
 	int num_of_channels = audio->getNumChannels();
-	int num_of_samples = audio->getNumSamples();
+	int num_of_samples = GetAudioSamplesCount();
 
 	// Resample to new sample rate (if needed)
 	if (new_sample_rate != sample_rate)
@@ -368,7 +385,7 @@ float* Frame::GetInterleavedAudioSamples(int new_sample_rate, AudioResampler* re
 	float *output = NULL;
 	AudioSampleBuffer *buffer(audio.get());
 	int num_of_channels = audio->getNumChannels();
-	int num_of_samples = audio->getNumSamples();
+	int num_of_samples = GetAudioSamplesCount();
 
 	// Resample to new sample rate (if needed)
 	if (new_sample_rate != sample_rate && resampler)
@@ -421,10 +438,7 @@ int Frame::GetAudioChannelsCount()
 int Frame::GetAudioSamplesCount()
 {
     const GenericScopedLock<CriticalSection> lock(addingAudioSection);
-	if (audio)
-		return audio->getNumSamples();
-	else
-		return 0;
+	return max_audio_sample;
 }
 
 juce::AudioSampleBuffer *Frame::GetAudioSampleBuffer()
@@ -453,7 +467,7 @@ const unsigned char* Frame::GetPixels()
 	// Check for blank image
 	if (!image)
 		// Fill with black
-		AddColor(width, height, "#000000");
+		AddColor(width, height, color);
 
 	// Return array of pixel packets
 	return image->bits();
@@ -498,6 +512,8 @@ int Frame::GetSamplesPerFrame(int64_t number, Fraction fps, int sample_rate, int
 	// Subtract the previous frame's total samples with this frame's total samples.  Not all sample rates can
 	// be evenly divided into frames, so each frame can have have different # of samples.
 	int samples_per_frame = round(total_samples - previous_samples);
+	if (samples_per_frame < 0)
+		samples_per_frame = 0;
 	return samples_per_frame;
 }
 
@@ -565,18 +581,16 @@ void Frame::Save(string path, float scale, string format, int quality)
 
 // Thumbnail the frame image to the specified path.  The image format is determined from the extension (i.e. image.PNG, image.JPEG)
 void Frame::Thumbnail(string path, int new_width, int new_height, string mask_path, string overlay_path,
-		string background_color, bool ignore_aspect, string format, int quality) {
+		string background_color, bool ignore_aspect, string format, int quality, float rotate) {
 
 	// Create blank thumbnail image & fill background color
 	std::shared_ptr<QImage> thumbnail = std::shared_ptr<QImage>(new QImage(new_width, new_height, QImage::Format_RGBA8888));
 	thumbnail->fill(QColor(QString::fromStdString(background_color)));
 
-	// Create transform and painter
-	QTransform transform;
+	// Create painter
 	QPainter painter(thumbnail.get());
 	painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform | QPainter::TextAntialiasing, true);
 
-
 	// Get preview image
 	std::shared_ptr<QImage> previewImage = GetImage();
 
@@ -603,6 +617,18 @@ void Frame::Thumbnail(string path, int new_width, int new_height, string mask_pa
 	int x = (new_width - previewImage->size().width()) / 2.0; // center
 	int y = (new_height - previewImage->size().height()) / 2.0; // center
 	painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
+
+
+	// Create transform and rotate (if needed)
+	QTransform transform;
+	float origin_x = previewImage->width() / 2.0;
+	float origin_y = previewImage->height() / 2.0;
+	transform.translate(origin_x, origin_y);
+	transform.rotate(rotate);
+	transform.translate(-origin_x,-origin_y);
+	painter.setTransform(transform);
+
+	// Draw image onto QImage
 	painter.drawImage(x, y, *previewImage);
 
 
@@ -678,8 +704,11 @@ int Frame::constrain(int color_value)
 }
 
 // Add (or replace) pixel data to the frame (based on a solid color)
-void Frame::AddColor(int new_width, int new_height, string color)
+void Frame::AddColor(int new_width, int new_height, string new_color)
 {
+	// Set color
+	color = new_color;
+
 	// Create new image object, and fill with pixel data
 	const GenericScopedLock<CriticalSection> lock(addingImageSection);
 	#pragma omp critical (AddImage)
@@ -713,7 +742,7 @@ void Frame::AddImage(int new_width, int new_height, int bytes_per_pixel, QImage:
 
 		// Always convert to RGBA8888 (if different)
 		if (image->format() != QImage::Format_RGBA8888)
-			image->convertToFormat(QImage::Format_RGBA8888);
+			*image  = image->convertToFormat(QImage::Format_RGBA8888);
 
 			// Update height and width
 		width = image->width();
@@ -737,7 +766,7 @@ void Frame::AddImage(std::shared_ptr<QImage> new_image)
 
 		// Always convert to RGBA8888 (if different)
 		if (image->format() != QImage::Format_RGBA8888)
-			image->convertToFormat(QImage::Format_RGBA8888);
+			*image = image->convertToFormat(QImage::Format_RGBA8888);
 
 		// Update height and width
 		width = image->width();
@@ -802,6 +831,9 @@ void Frame::ResizeAudio(int channels, int length, int rate, ChannelLayout layout
 	audio->setSize(channels, length, true, true, false);
 	channel_layout = layout;
 	sample_rate = rate;
+
+	// Calculate max audio sample added
+	max_audio_sample = length;
 }
 
 // Add audio samples to a specific channel
@@ -809,8 +841,11 @@ void Frame::AddAudio(bool replaceSamples, int destChannel, int destStartSample,
 	const GenericScopedLock<CriticalSection> lock(addingAudioSection);
 	#pragma omp critical (adding_audio)
     {
+		// Clamp starting sample to 0
+		int destStartSampleAdjusted = max(destStartSample, 0);
+
 		// Extend audio container to hold more (or less) samples and channels.. if needed
-		int new_length = destStartSample + numSamples;
+		int new_length = destStartSampleAdjusted + numSamples;
 		int new_channel_length = audio->getNumChannels();
 		if (destChannel >= new_channel_length)
 			new_channel_length = destChannel + 1;
@@ -819,11 +854,15 @@ void Frame::AddAudio(bool replaceSamples, int destChannel, int destStartSample,
 
 		// Clear the range of samples first (if needed)
 		if (replaceSamples)
-			audio->clear(destChannel, destStartSample, numSamples);
+			audio->clear(destChannel, destStartSampleAdjusted, numSamples);
 
 		// Add samples to frame's audio buffer
-		audio->addFrom(destChannel, destStartSample, source, numSamples, gainToApplyToSource);
+		audio->addFrom(destChannel, destStartSampleAdjusted, source, numSamples, gainToApplyToSource);
 		has_audio_data = true;
+
+		// Calculate max audio sample added
+		if (new_length > max_audio_sample)
+			max_audio_sample = new_length;
 	}
 }
 
@@ -842,7 +881,7 @@ std::shared_ptr<QImage> Frame::GetImage()
 	// Check for blank image
 	if (!image)
 		// Fill with black
-		AddColor(width, height, "#000000");
+		AddColor(width, height, color);
 
 	return image;
 }
@@ -910,7 +949,7 @@ void Frame::AddMagickImage(std::shared_ptr<Magick::Image> new_image)
 void Frame::Play()
 {
 	// Check if samples are present
-	if (!audio->getNumSamples())
+	if (!GetAudioSamplesCount())
 		return;
 
 	AudioDeviceManager deviceManager;
@@ -992,4 +1031,8 @@ void Frame::AddAudioSilence(int numSamples)
 	audio->setSize(channels, numSamples, false, true, false);
 	audio->clear();
 	has_audio_data = true;
+
+	// Calculate max audio sample added
+	if (numSamples > max_audio_sample)
+		max_audio_sample = numSamples;
 }
