@@ -127,16 +127,6 @@ void Timeline::apply_mapper_to_clip(Clip* clip)
 	FrameMapper* clip_mapped_reader = (FrameMapper*) clip_reader;
 	clip_mapped_reader->ChangeMapping(info.fps, PULLDOWN_NONE, info.sample_rate, info.channels, info.channel_layout);
 
-	// Update timeline offset
-	float time_diff = 0 - clip->Position() + clip->Start();
-	int clip_offset = -round(time_diff * info.fps.ToFloat());
-
-	if (clip_offset != 0)
-		// Reduce negative offset by 1 (since we want to avoid frame 0)
-		clip_offset += 1;
-
-	clip_mapped_reader->SetTimelineFrameOffset(clip_offset);
-
 	// Update clip reader
 	clip->Reader(clip_reader);
 }
@@ -255,7 +245,7 @@ std::shared_ptr<Frame> Timeline::GetOrCreateFrame(Clip* clip, int64_t number)
 }
 
 // Process a new layer of video or audio
-void Timeline::add_layer(std::shared_ptr<Frame> new_frame, Clip* source_clip, int64_t clip_frame_number, int64_t timeline_frame_number, bool is_top_clip)
+void Timeline::add_layer(std::shared_ptr<Frame> new_frame, Clip* source_clip, int64_t clip_frame_number, int64_t timeline_frame_number, bool is_top_clip, float max_volume)
 {
 	// Get the clip's frame & image
 	std::shared_ptr<Frame> source_frame;
@@ -291,6 +281,7 @@ void Timeline::add_layer(std::shared_ptr<Frame> new_frame, Clip* source_clip, in
 	/* Apply effects to the source frame (if any). If multiple clips are overlapping, only process the
 	 * effects on the top clip. */
 	if (is_top_clip && source_frame)
+		#pragma omp critical (T_addLayer)
 		source_frame = apply_effects(source_frame, timeline_frame_number, source_clip->Layer());
 
 	// Declare an image to hold the source frame's image
@@ -298,33 +289,44 @@ void Timeline::add_layer(std::shared_ptr<Frame> new_frame, Clip* source_clip, in
 
 	/* COPY AUDIO - with correct volume */
 	if (source_clip->Reader()->info.has_audio) {
-
 		// Debug output
 		ZmqLogger::Instance()->AppendDebugMethod("Timeline::add_layer (Copy Audio)", "source_clip->Reader()->info.has_audio", source_clip->Reader()->info.has_audio, "source_frame->GetAudioChannelsCount()", source_frame->GetAudioChannelsCount(), "info.channels", info.channels, "clip_frame_number", clip_frame_number, "timeline_frame_number", timeline_frame_number, "", -1);
 
-		if (source_frame->GetAudioChannelsCount() == info.channels)
+		if (source_frame->GetAudioChannelsCount() == info.channels && source_clip->has_audio.GetInt(clip_frame_number) != 0)
 			for (int channel = 0; channel < source_frame->GetAudioChannelsCount(); channel++)
 			{
-				float initial_volume = 1.0f;
-				float previous_volume = source_clip->volume.GetValue(clip_frame_number - 1); // previous frame's percentage of volume (0 to 1)
-				float volume = source_clip->volume.GetValue(clip_frame_number); // percentage of volume (0 to 1)
+				// Get volume from previous frame and this frame
+				float previous_volume = source_clip->volume.GetValue(clip_frame_number - 1);
+				float volume = source_clip->volume.GetValue(clip_frame_number);
 				int channel_filter = source_clip->channel_filter.GetInt(clip_frame_number); // optional channel to filter (if not -1)
 				int channel_mapping = source_clip->channel_mapping.GetInt(clip_frame_number); // optional channel to map this channel to (if not -1)
 
+				// Apply volume mixing strategy
+				if (source_clip->mixing == VOLUME_MIX_AVERAGE && max_volume > 1.0) {
+					// Don't allow this clip to exceed 100% (divide volume equally between all overlapping clips with volume
+					previous_volume = previous_volume / max_volume;
+					volume = volume / max_volume;
+				}
+				else if (source_clip->mixing == VOLUME_MIX_REDUCE && max_volume > 1.0) {
+					// Reduce clip volume by a bit, hoping it will prevent exceeding 100% (but it is very possible it will)
+					previous_volume = previous_volume * 0.77;
+					volume = volume * 0.77;
+				}
+
 				// If channel filter enabled, check for correct channel (and skip non-matching channels)
 				if (channel_filter != -1 && channel_filter != channel)
 					continue; // skip to next channel
 
+				// If no volume on this frame or previous frame, do nothing
+				if (previous_volume == 0.0 && volume == 0.0)
+					continue; // skip to next channel
+
 				// If channel mapping disabled, just use the current channel
 				if (channel_mapping == -1)
 					channel_mapping = channel;
 
-				// If no ramp needed, set initial volume = clip's volume
-				if (isEqual(previous_volume, volume))
-					initial_volume = volume;
-
 				// Apply ramp to source frame (if needed)
-				if (!isEqual(previous_volume, volume))
+				if (!isEqual(previous_volume, 1.0) || !isEqual(volume, 1.0))
 					source_frame->ApplyGainRamp(channel_mapping, 0, source_frame->GetAudioSamplesCount(), previous_volume, volume);
 
 				// TODO: Improve FrameMapper (or Timeline) to always get the correct number of samples per frame.
@@ -339,7 +341,7 @@ void Timeline::add_layer(std::shared_ptr<Frame> new_frame, Clip* source_clip, in
 				// Copy audio samples (and set initial volume).  Mix samples with existing audio samples.  The gains are added together, to
 				// be sure to set the gain's correctly, so the sum does not exceed 1.0 (of audio distortion will happen).
 				#pragma omp critical (T_addLayer)
-				new_frame->AddAudio(false, channel_mapping, 0, source_frame->GetAudioSamples(channel), source_frame->GetAudioSamplesCount(), initial_volume);
+				new_frame->AddAudio(false, channel_mapping, 0, source_frame->GetAudioSamples(channel), source_frame->GetAudioSamplesCount(), 1.0);
 
 			}
 		else
@@ -575,8 +577,13 @@ void Timeline::update_open_clips(Clip *clip, bool does_clip_intersect)
 		// Add clip to 'opened' list, because it's missing
 		open_clips[clip] = clip;
 
-		// Open the clip
-		clip->Open();
+		try {
+			// Open the clip
+			clip->Open();
+
+		} catch (const InvalidFile & e) {
+			// ...
+		}
 	}
 
 	// Debug output
@@ -716,7 +723,7 @@ std::shared_ptr<Frame> Timeline::GetFrame(int64_t requested_frame)
 		#pragma omp parallel
 		{
 			// Loop through all requested frames
-			#pragma omp for ordered firstprivate(nearby_clips, requested_frame, minimum_frames)
+			#pragma omp for ordered firstprivate(nearby_clips, requested_frame, minimum_frames) schedule(static,1)
 			for (int64_t frame_number = requested_frame; frame_number < requested_frame + minimum_frames; frame_number++)
 			{
 				// Debug output
@@ -763,17 +770,27 @@ std::shared_ptr<Frame> Timeline::GetFrame(int64_t requested_frame)
 					{
 						// Determine if clip is "top" clip on this layer (only happens when multiple clips are overlapping)
 						bool is_top_clip = true;
+						float max_volume = 0.0;
 						for (int top_clip_index = 0; top_clip_index < nearby_clips.size(); top_clip_index++)
 						{
 							Clip *nearby_clip = nearby_clips[top_clip_index];
                             long nearby_clip_start_position = round(nearby_clip->Position() * info.fps.ToDouble()) + 1;
                             long nearby_clip_end_position = round((nearby_clip->Position() + nearby_clip->Duration()) * info.fps.ToDouble()) + 1;
+							long nearby_clip_start_frame = (nearby_clip->Start() * info.fps.ToDouble()) + 1;
+							long nearby_clip_frame_number = frame_number - nearby_clip_start_position + nearby_clip_start_frame;
 
+							// Determine if top clip
 							if (clip->Id() != nearby_clip->Id() && clip->Layer() == nearby_clip->Layer() &&
                                     nearby_clip_start_position <= frame_number && nearby_clip_end_position >= frame_number &&
-                                    nearby_clip_start_position > clip_start_position) {
+                                    nearby_clip_start_position > clip_start_position && is_top_clip == true) {
 								is_top_clip = false;
-								break;
+							}
+
+							// Determine max volume of overlapping clips
+							if (nearby_clip->Reader() && nearby_clip->Reader()->info.has_audio &&
+									nearby_clip->has_audio.GetInt(nearby_clip_frame_number) != 0 &&
+									nearby_clip_start_position <= frame_number && nearby_clip_end_position >= frame_number) {
+									max_volume += nearby_clip->volume.GetValue(nearby_clip_frame_number);
 							}
 						}
 
@@ -785,7 +802,7 @@ std::shared_ptr<Frame> Timeline::GetFrame(int64_t requested_frame)
 						ZmqLogger::Instance()->AppendDebugMethod("Timeline::GetFrame (Calculate clip's frame #)", "clip->Position()", clip->Position(), "clip->Start()", clip->Start(), "info.fps.ToFloat()", info.fps.ToFloat(), "clip_frame_number", clip_frame_number, "", -1, "", -1);
 
 						// Add clip's frame as layer
-						add_layer(new_frame, clip, clip_frame_number, frame_number, is_top_clip);
+						add_layer(new_frame, clip, clip_frame_number, frame_number, is_top_clip, max_volume);
 
 					} else
 						// Debug output
@@ -797,7 +814,7 @@ std::shared_ptr<Frame> Timeline::GetFrame(int64_t requested_frame)
 				ZmqLogger::Instance()->AppendDebugMethod("Timeline::GetFrame (Add frame to cache)", "frame_number", frame_number, "info.width", info.width, "info.height", info.height, "", -1, "", -1, "", -1);
 
 				// Set frame # on mapped frame
-				#pragma omp critical (T_GetFrame)
+				#pragma omp ordered
 				{
 					new_frame->SetFrameNumber(frame_number);
 
@@ -989,13 +1006,14 @@ void Timeline::SetJsonValue(Json::Value root) {
 
 			if (!existing_effect["type"].isNull()) {
 				// Create instance of effect
-				e = EffectInfo().CreateEffect(existing_effect["type"].asString());
+				if (e = EffectInfo().CreateEffect(existing_effect["type"].asString())) {
 
-				// Load Json into Effect
-				e->SetJsonValue(existing_effect);
+					// Load Json into Effect
+					e->SetJsonValue(existing_effect);
 
-				// Add Effect to Timeline
-				AddEffect(e);
+					// Add Effect to Timeline
+					AddEffect(e);
+				}
 			}
 		}
 	}
@@ -1142,6 +1160,9 @@ void Timeline::apply_json_to_clips(Json::Value change) {
 		clip->SetJsonValue(change["value"]); // Set properties of new clip from JSON
 		AddClip(clip); // Add clip to timeline
 
+		// Apply framemapper (or update existing framemapper)
+		apply_mapper_to_clip(clip);
+
 	} else if (change_type == "update") {
 
 		// Update existing clip
@@ -1159,6 +1180,9 @@ void Timeline::apply_json_to_clips(Json::Value change) {
 			// Update clip properties from JSON
 			existing_clip->SetJsonValue(change["value"]);
 
+			// Apply framemapper (or update existing framemapper)
+			apply_mapper_to_clip(existing_clip);
+
             // Clear any cached image sizes (since size might have changed)
             existing_clip->SetMaxSize(0, 0); // force clearing of cached image size
             if (existing_clip->Reader()) {
@@ -1253,13 +1277,14 @@ void Timeline::apply_json_to_effects(Json::Value change, EffectBase* existing_ef
 		EffectBase *e = NULL;
 
 		// Init the matching effect object
-		e = EffectInfo().CreateEffect(effect_type);
+		if (e = EffectInfo().CreateEffect(effect_type)) {
 
-		// Load Json into Effect
-		e->SetJsonValue(change["value"]);
+			// Load Json into Effect
+			e->SetJsonValue(change["value"]);
 
-		// Add Effect to Timeline
-		AddEffect(e);
+			// Add Effect to Timeline
+			AddEffect(e);
+		}
 
 	} else if (change_type == "update") {
 
