@@ -56,19 +56,6 @@ void Brightness::init_effect_details()
 	info.has_video = true;
 }
 
-// Constrain a color value from 0 to 255
-int Brightness::constrain(int color_value)
-{
-	// Constrain new color from 0 to 255
-	if (color_value < 0)
-		color_value = 0;
-	else if (color_value > 255)
-		color_value = 255;
-
-	return color_value;
-}
-
-
 // This method is required for all derived classes of EffectBase, and returns a
 // modified openshot::Frame object
 std::shared_ptr<Frame> Brightness::GetFrame(std::shared_ptr<Frame> frame, int64_t frame_number)
@@ -91,7 +78,7 @@ std::shared_ptr<Frame> Brightness::GetFrame(std::shared_ptr<Frame> frame, int64_
 		int A = pixels[byte_index + 3];
 
 		// Adjust the contrast
-		int factor = (259 * (contrast_value + 255)) / (255 * (259 - contrast_value));
+		float factor = (259 * (contrast_value + 255)) / (255 * (259 - contrast_value));
 		R = constrain((factor * (R - 128)) + 128);
 		G = constrain((factor * (G - 128)) + 128);
 		B = constrain((factor * (B - 128)) + 128);
