@@ -30,14 +30,14 @@
 using namespace openshot;
 
 /// Blank constructor, useful when using Json to load the effect properties
-Mask::Mask() : reader(NULL), replace_image(false) {
+Mask::Mask() : reader(NULL), replace_image(false), needs_refresh(true) {
 	// Init effect properties
 	init_effect_details();
 }
 
 // Default constructor
 Mask::Mask(ReaderBase *mask_reader, Keyframe mask_brightness, Keyframe mask_contrast) :
-		reader(mask_reader), brightness(mask_brightness), contrast(mask_contrast), replace_image(false)
+		reader(mask_reader), brightness(mask_brightness), contrast(mask_contrast), replace_image(false), needs_refresh(true)
 {
 	// Init effect properties
 	init_effect_details();
@@ -57,109 +57,91 @@ void Mask::init_effect_details()
 	info.has_video = true;
 }
 
-// Constrain a color value from 0 to 255
-int Mask::constrain(int color_value)
-{
-	// Constrain new color from 0 to 255
-	if (color_value < 0)
-		color_value = 0;
-	else if (color_value > 255)
-		color_value = 255;
-
-	return color_value;
-}
-
-// Get grayscale mask image
-void Mask::set_grayscale_mask(std::shared_ptr<QImage> mask_frame_image, int width, int height, float brightness, float contrast)
-{
-	// Get pixels for mask image
-	unsigned char *pixels = (unsigned char *) mask_frame_image->bits();
-
-	// Convert the mask image to grayscale
-	// Loop through pixels
-	for (int pixel = 0, byte_index=0; pixel < mask_frame_image->width() * mask_frame_image->height(); pixel++, byte_index+=4)
-	{
-		// Get the RGB values from the pixel
-		int R = pixels[byte_index];
-		int G = pixels[byte_index + 1];
-		int B = pixels[byte_index + 2];
-
-		// Get the average luminosity
-		int gray_value = qGray(R, G, B);
-
-		// Adjust the contrast
-		int factor = (259 * (contrast + 255)) / (255 * (259 - contrast));
-		gray_value = constrain((factor * (gray_value - 128)) + 128);
-
-		// Adjust the brightness
-		gray_value += (255 * brightness);
-
-		// Constrain the value from 0 to 255
-		gray_value = constrain(gray_value);
-
-		// Set all pixels to gray value
-		pixels[byte_index] = gray_value;
-		pixels[byte_index + 1] = gray_value;
-		pixels[byte_index + 2] = gray_value;
-		pixels[byte_index + 3] = 255;
-	}
-}
-
 // This method is required for all derived classes of EffectBase, and returns a
 // modified openshot::Frame object
-std::shared_ptr<Frame> Mask::GetFrame(std::shared_ptr<Frame> frame, int64_t frame_number)
-{
+std::shared_ptr<Frame> Mask::GetFrame(std::shared_ptr<Frame> frame, int64_t frame_number) {
 	// Get the mask image (from the mask reader)
 	std::shared_ptr<QImage> frame_image = frame->GetImage();
 
 	// Check if mask reader is open
-	if (reader && !reader->IsOpen())
-		#pragma omp critical (open_mask_reader)
-		reader->Open();
+	#pragma omp critical (open_mask_reader)
+	{
+		if (reader && !reader->IsOpen())
+			reader->Open();
+	}
 
 	// No reader (bail on applying the mask)
 	if (!reader)
 		return frame;
 
 	// Get mask image (if missing or different size than frame image)
-	if (!original_mask || !reader->info.has_single_image ||
-            (original_mask && original_mask->size() != frame_image->size())) {
-		#pragma omp critical (open_mask_reader)
-		{
+	#pragma omp critical (open_mask_reader)
+	{
+		if (!original_mask || !reader->info.has_single_image || needs_refresh ||
+			(original_mask && original_mask->size() != frame_image->size())) {
+
 			// Only get mask if needed
-			std::shared_ptr<QImage> mask_without_sizing = std::shared_ptr<QImage>(new QImage(*reader->GetFrame(frame_number)->GetImage()));
+			std::shared_ptr<QImage> mask_without_sizing = std::shared_ptr<QImage>(
+					new QImage(*reader->GetFrame(frame_number)->GetImage()));
 
 			// Resize mask image to match frame size
 			original_mask = std::shared_ptr<QImage>(new QImage(
 					mask_without_sizing->scaled(frame_image->width(), frame_image->height(), Qt::IgnoreAspectRatio,
-										  Qt::SmoothTransformation)));
+												Qt::SmoothTransformation)));
 		}
 	}
 
-	// Convert mask to grayscale and resize to frame size
-	std::shared_ptr<QImage> mask = std::shared_ptr<QImage>(new QImage(*original_mask));
-	set_grayscale_mask(mask, frame_image->width(), frame_image->height(), brightness.GetValue(frame_number), contrast.GetValue(frame_number));
+	// Refresh no longer needed
+	needs_refresh = false;
 
-	// Get pixels for frame image
+	// Get pixel arrays
 	unsigned char *pixels = (unsigned char *) frame_image->bits();
-	unsigned char *mask_pixels = (unsigned char *) mask->bits();
+	unsigned char *mask_pixels = (unsigned char *) original_mask->bits();
 
-	// Convert the mask image to grayscale
-	// Loop through pixels
-	for (int pixel = 0, byte_index=0; pixel < frame_image->width() * frame_image->height(); pixel++, byte_index+=4)
+	int R = 0;
+	int G = 0;
+	int B = 0;
+	int A = 0;
+	int gray_value = 0;
+	float factor = 0.0;
+	double contrast_value = (contrast.GetValue(frame_number));
+	double brightness_value = (brightness.GetValue(frame_number));
+
+	// Loop through mask pixels, and apply average gray value to frame alpha channel
+	for (int pixel = 0, byte_index=0; pixel < original_mask->width() * original_mask->height(); pixel++, byte_index+=4)
 	{
 		// Get the RGB values from the pixel
-		int Frame_Alpha = pixels[byte_index + 3];
-		int Mask_Value = constrain(Frame_Alpha - (int)mask_pixels[byte_index]); // Red pixel (all colors should have the same value here)
+		R = mask_pixels[byte_index];
+		G = mask_pixels[byte_index + 1];
+		B = mask_pixels[byte_index + 2];
+
+		// Get the average luminosity
+		gray_value = qGray(R, G, B);
+
+		// Adjust the contrast
+		factor = (259 * (contrast_value + 255)) / (255 * (259 - contrast_value));
+		gray_value = constrain((factor * (gray_value - 128)) + 128);
+
+		// Adjust the brightness
+		gray_value += (255 * brightness_value);
+
+		// Constrain the value from 0 to 255
+		gray_value = constrain(gray_value);
+
+		// Set the alpha channel to the gray value
+		if (replace_image) {
+			// Replace frame pixels with gray value
+			pixels[byte_index + 0] = gray_value;
+			pixels[byte_index + 1] = gray_value;
+			pixels[byte_index + 2] = gray_value;
+		} else {
+			// Set alpha channel
+			A = pixels[byte_index + 3];
+			pixels[byte_index + 3] = constrain(A - gray_value);
+		}
 
-		// Set all pixels to gray value
-		pixels[byte_index + 3] = Mask_Value;
 	}
 
-	// Replace the frame's image with the current mask (good for debugging)
-	if (replace_image)
-		frame->AddImage(mask); // not typically called when using a mask
-
 	// return the modified frame
 	return frame;
 }
@@ -227,47 +209,51 @@ void Mask::SetJsonValue(Json::Value root) {
 		contrast.SetJsonValue(root["contrast"]);
 	if (!root["reader"].isNull()) // does Json contain a reader?
 	{
-
-		if (!root["reader"]["type"].isNull()) // does the reader Json contain a 'type'?
+		#pragma omp critical (open_mask_reader)
 		{
-			// Close previous reader (if any)
-			if (reader)
+			// This reader has changed, so refresh cached assets
+			needs_refresh = true;
+
+			if (!root["reader"]["type"].isNull()) // does the reader Json contain a 'type'?
 			{
-				// Close and delete existing reader (if any)
-				reader->Close();
-				delete reader;
-				reader = NULL;
-			}
+				// Close previous reader (if any)
+				if (reader) {
+					// Close and delete existing reader (if any)
+					reader->Close();
+					delete reader;
+					reader = NULL;
+				}
 
-			// Create new reader (and load properties)
-			string type = root["reader"]["type"].asString();
+				// Create new reader (and load properties)
+				string type = root["reader"]["type"].asString();
 
-			if (type == "FFmpegReader") {
+				if (type == "FFmpegReader") {
 
-				// Create new reader
-				reader = new FFmpegReader(root["reader"]["path"].asString());
-				reader->SetJsonValue(root["reader"]);
+					// Create new reader
+					reader = new FFmpegReader(root["reader"]["path"].asString());
+					reader->SetJsonValue(root["reader"]);
 
-#ifdef USE_IMAGEMAGICK
-			} else if (type == "ImageReader") {
+	#ifdef USE_IMAGEMAGICK
+					} else if (type == "ImageReader") {
 
-				// Create new reader
-				reader = new ImageReader(root["reader"]["path"].asString());
-				reader->SetJsonValue(root["reader"]);
-#endif
+						// Create new reader
+						reader = new ImageReader(root["reader"]["path"].asString());
+						reader->SetJsonValue(root["reader"]);
+	#endif
 
-			} else if (type == "QtImageReader") {
+				} else if (type == "QtImageReader") {
 
-				// Create new reader
-				reader = new QtImageReader(root["reader"]["path"].asString());
-				reader->SetJsonValue(root["reader"]);
+					// Create new reader
+					reader = new QtImageReader(root["reader"]["path"].asString());
+					reader->SetJsonValue(root["reader"]);
 
-			} else if (type == "ChunkReader") {
+				} else if (type == "ChunkReader") {
 
-				// Create new reader
-				reader = new ChunkReader(root["reader"]["path"].asString(), (ChunkVersion) root["reader"]["chunk_version"].asInt());
-				reader->SetJsonValue(root["reader"]);
+					// Create new reader
+					reader = new ChunkReader(root["reader"]["path"].asString(), (ChunkVersion) root["reader"]["chunk_version"].asInt());
+					reader->SetJsonValue(root["reader"]);
 
+				}
 			}
 
 		}
@@ -296,6 +282,11 @@ string Mask::PropertiesJSON(int64_t requested_frame) {
 	root["brightness"] = add_property_json("Brightness", brightness.GetValue(requested_frame), "float", "", &brightness, -1.0, 1.0, false, requested_frame);
 	root["contrast"] = add_property_json("Contrast", contrast.GetValue(requested_frame), "float", "", &contrast, 0, 20, false, requested_frame);
 
+	if (reader)
+		root["reader"] = add_property_json("Source", 0.0, "reader", reader->Json(), NULL, 0, 1, false, requested_frame);
+	else
+		root["reader"] = add_property_json("Source", 0.0, "reader", "{}", NULL, 0, 1, false, requested_frame);
+
 	// Return formatted string
 	return root.toStyledString();
 }
